@@ -19,10 +19,14 @@ __license__ = "Apache 2.0"
 
 import csv
 import sys
+import pendulum
 import json
 import yaml
 import requests
+import shapely
 import clipboard
+import visvalingamwyatt as vw
+from shapely.geometry import shape
 from prettytable import PrettyTable
 from planet.api.utils import read_planet_json
 from planet.api.auth import find_api_key
@@ -30,12 +34,27 @@ from planet.api.auth import find_api_key
 
 x = PrettyTable()
 
+ovall=[]
+
+# get coordinates list depth
+def list_depth(dic, level = 1):
+    counter = 0
+    str_dic = str(dic)
+    if "[[[[[" in str_dic:
+        counter += 2
+    elif "[[[[" in str_dic:
+        counter += 1
+    elif "[[[" in str_dic:
+        counter += 0
+    return(counter)
+
 ##Setup for bundles
-dbundle = {'name': [], 'order_type': 'partial', 'products': [{'item_ids': [], 'item_type': [],'product_bundle': []}],'tools':[]}
-dclip = {"clip": {"aoi": {"type": "Polygon","coordinates": []}}}
+dbundle = {'name': [], 'subscription_id':[], 'order_type': 'partial', 'products': [{'item_ids': [], 'item_type': [],'product_bundle': []}],'tools':[], 'delivery':{}}
+dclip = {"clip": {"aoi": {"type": "MultiPolygon","coordinates": []}}}
 dtoar = {'toar': {'scale_factor': 10000}}
 dzip = {"delivery":{"archive_filename":"{{name}}.zip","archive_type":"zip"}}
 dcomposite ={"composite":{}}
+dharmonize={"harmonize": {"target_sensor": "PS2"}}
 dreproject={"reproject": {"projection": [],"kernel":[]}}
 dtiff={"tiff_optimize": {"compression": []}}
 demail={'notifications':{'email': True}}
@@ -43,7 +62,7 @@ daws= {"delivery":{"amazon_s3":{"bucket":[],"aws_region":[],"aws_access_key_id":
 dazure={"delivery":{"azure_blob_storage":{"account":[],"container":[],"sas_token":[],"storage_endpoint_suffix":[],"path_prefix":[]}}}
 dgcs={"delivery": {"google_cloud_storage": {"bucket": [],"credentials": [],"path_prefix": []}}}
 dbmath={"bandmath":{}}
-
+dszip={"delivery":{"archive_filename":"Explorer_{{name}}.zip","archive_type":"zip","single_archive":True}}
 try:
     PL_API_KEY = find_api_key()
 except:
@@ -51,10 +70,31 @@ except:
     sys.exit()
 url = 'https://api.planet.com/compute/ops/orders/v2'
 
+def vdepth(dic, level=1):
+    counter = 0
+    str_dic = str(dic)
+    if "[[[[" in str_dic:
+        counter += 1
+    return(counter)
+
+def vertexcount(input):
+        aoi_resp = input
+        if vdepth(aoi_resp['geometry']['coordinates'])==0:
+            return str(len(aoi_resp['geometry']['coordinates'][0]))
+        elif vdepth(aoi_resp['geometry']['coordinates'])==1:
+            return str(len(aoi_resp['geometry']['coordinates'][0][0]))
+        else:
+            print('Please check GeoJSON: Could not parse coordinates')
+
 def order(**kwargs):
     for key,value in kwargs.items():
         if key=='name':
             dbundle['name']=value
+        if key=='sid':
+            if value is not None:
+                dbundle['subscription_id']=int(value)
+            else:
+                dbundle.pop('subscription_id',None)
         if key=='item':
             dbundle['products'][0]['item_type']=value
         if key=='asset':
@@ -85,19 +125,23 @@ def order(**kwargs):
                     dbundle['tools'].append(dclip)
                 elif items=='toar':
                     dbundle['tools'].append(dtoar)
+                elif items=='harmonize':
+                    dbundle['tools'].append(dharmonize)
                 elif items=='zip':
-                    dbundle.update(dzip)
+                    dbundle['delivery'].update(dzip['delivery'])
+                elif items=='zipall':
+                    dbundle['delivery'].update(dszip['delivery'])
                 elif items=='email':
                     dbundle.update(demail)
                 elif items=='aws':
-                    dbundle.update(daws)
+                    dbundle['delivery'].update(daws['delivery'])
                 elif items=='azure':
-                    dbundle.update(dazure)
+                    dbundle['delivery'].update(dazure['delivery'])
                 elif items=='gcs':
-                    dbundle.update(dgcs)
+                    dbundle['delivery'].update(dgcs['delivery'])
                 elif items=='composite':
                     dbundle['tools'].append(dcomposite)
-                elif items=='reproject':
+                elif items=='projection':
                     dbundle['tools'].append(dreproject)
                 elif items=='ndvi':
                     dndvi={"pixel_type": "32R","ndvi": "(b4 - b3) / (b4+b3)"}
@@ -128,7 +172,7 @@ def order(**kwargs):
                     dbmath['bandmath'].update(dmsavi2)
                 elif items=='compression':
                     dbundle['tools'].append(dtiff)
-
+            #dbundle[]
     for key,value in kwargs.items():
         if key=='boundary' and value!=None:
                 for items in k['tools']:
@@ -136,8 +180,49 @@ def order(**kwargs):
                         try:
                             if value.endswith('.geojson'):
                                 with open(value) as aoi:
-                                    aoi_resp = json.loads(aoi.read())
-                                    items['clip']['aoi']['coordinates']= aoi_resp['features'][0]['geometry']['coordinates']
+                                    aoi_resp = json.load(aoi)
+                                    for things in aoi_resp['features']:
+                                        ovall.append(things['geometry']['coordinates'])
+                                #print(list_depth(ovall))
+                                if len(ovall)>1:
+                                    aoi_geom=ovall
+                                    items['clip']['aoi']['coordinates']=aoi_geom
+                                    temp=shape(items.get('clip')['aoi'])
+                                    if not temp.is_valid:
+                                        print('Solving shapely self intersection for multipolygon')
+                                        temp=temp.buffer(0)
+                                        geojson_string = json.dumps(shapely.geometry.mapping(temp))
+                                        items['clip']['aoi']['coordinates']=json.loads(geojson_string)['coordinates']
+                                        ft = {"type": "Feature","geometry": {"type": "Polygon","coordinates": []},"properties": {}}
+                                        ft['geometry']['coordinates']=json.loads(geojson_string)['coordinates']
+                                        print('Total number of vertices in geometry: '+str(vertexcount(ft)))
+                                        if int(vertexcount(ft))>500:
+                                            print('Simplifying geometry since Ordersv2 will only accept upto 500 vertex points')
+                                            b= vw.simplify_feature(ft, number=495)
+                                            print('Total number of vertices in simplified geometry: '+str(vertexcount(b))+'\n')
+                                            items['clip']['aoi']['coordinates']=b['geometry']['coordinates']
+                                            temp=shape(b['geometry'])
+                                            if not temp.is_valid:
+                                                temp=temp.buffer(0)
+                                                geojson_string = json.dumps(shapely.geometry.mapping(temp))
+                                                items['clip']['aoi']['coordinates']=json.loads(geojson_string)['coordinates']
+                                            elif temp.is_valid:
+                                                items['clip']['aoi']['coordinates']=b['geometry']['coordinates']
+                                else:
+                                    if list_depth(ovall)==0:
+                                        aoi_geom = ovall
+                                        items['clip']['aoi']['type']="Polygon"
+                                        items['clip']['aoi']['coordinates']=aoi_geom
+                                    elif list_depth(ovall)==1:
+                                        aoi_geom = ovall[0]
+                                        items['clip']['aoi']['type']="Polygon"
+                                        items['clip']['aoi']['coordinates']=aoi_geom
+                                    elif list_depth(ovall)==2:
+                                        aoi_geom = ovall[0][0]
+                                        items['clip']['aoi']['type']="Polygon"
+                                        items['clip']['aoi']['coordinates']=aoi_geom
+                                    else:
+                                        print('Please check GeoJSON: Could not parse coordinates')
                             elif value.endswith('.json'):
                                 with open (value) as aoi:
                                     aoi_resp=json.load(aoi)
@@ -146,12 +231,13 @@ def order(**kwargs):
                                 getcoord=kml2coord(value)
                                 items['clip']['aoi']['coordinates']=getcoord
                         except Exception as e:
+                            #print(e)
                             print('Could not parse geometry')
         #         #print(e)
     for key,value in kwargs.items():
         if key=='aws' and value!=None:
             with open(value, 'r') as ymlfile:
-                cfg = yaml.load(ymlfile)
+                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                 for section in cfg:
                     k['delivery']['amazon_s3']['bucket']=cfg['amazon_s3']['bucket']
                     k['delivery']['amazon_s3']['aws_region']=cfg['amazon_s3']['aws_region']
@@ -161,7 +247,7 @@ def order(**kwargs):
     for key,value in kwargs.items():
         if key=='azure' and value!=None:
             with open(value, 'r') as ymlfile:
-                cfg = yaml.load(ymlfile)
+                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                 for section in cfg:
                     k['delivery']['azure_blob_storage']['account']=cfg['azure']['account']
                     k['delivery']['azure_blob_storage']['container']=cfg['azure']['container']
@@ -171,7 +257,7 @@ def order(**kwargs):
     for key,value in kwargs.items():
         if key=='gcs' and value!=None:
             with open(value, 'r') as ymlfile:
-                cfg = yaml.load(ymlfile)
+                cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)
                 for section in cfg:
                     k['delivery']['google_cloud_storage']['bucket']=cfg['gcs']['bucket']
                     k['delivery']['google_cloud_storage']['credentials']=cfg['gcs']['credentials']
@@ -224,6 +310,8 @@ def order(**kwargs):
     #print(dbmath)
 
     #print(payload)
+    ordname=k['name']
+    payload=payload.replace("Explorer_{{name}}.zip",ordname+'_'+str(pendulum.now()).split("T")[0]+".zip")
     headers = {'content-type': 'application/json',
                'cache-control': 'no-cache'}
     response = requests.request('POST', url, data=payload, headers=headers,
@@ -233,7 +321,15 @@ def order(**kwargs):
         try:
             clipboard.copy(str(url) + '/' + str(content['id']))
             print('Order created at '+str(url) + '/' + str(content['id']+' and url copied to clipboard'))
+            return (str(url) + '/' + str(content['id']))
         except Exception:
             print('Headless Setup: Order created at '+str(url) + '/' + str(content['id']))
+    elif response.status_code==400:
+        print('Failed with response: Bad request')
+        print(response.json()['general'][0]['message'])
+    elif response.status_code==401:
+        print('Failed with response: Forbidden')
+    elif response.status_code==409:
+        print('Failed with response: MaxConcurrency')
     else:
         print(response.text)
