@@ -22,11 +22,12 @@ def idsplit(infile,linenum,output):
     lines_per_file = int(linenum)
     smallfile = None
     with open(infile) as bigfile:
+        basename=os.path.basename(infile).split('.')[0]
         for lineno, line in enumerate(bigfile):
             if lineno % lines_per_file == 0:
                 if smallfile:
                     smallfile.close()
-                small_filename = 'link_file_{}.csv'.format(str(lineno+int(linenum)))
+                small_filename = basename+'_{}.csv'.format(str(lineno+int(linenum)))
                 smallfile = open(os.path.join(output,small_filename), "w")
             smallfile.write(line)
         if smallfile:
